/* Copyright (C) 2023 FIAS Frankfurt Institute for Advanced Studies, Frankfurt / Main
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Felix Weiglhofer [committer] */
#ifndef CBM_BASE_TYPES_H
#define CBM_BASE_TYPES_H

#include "MicrosliceDescriptor.hpp"  // For fles::SubsystemIdentifier

#include "util/EnumDict.h"

namespace cbm::algo
{

  enum class Step
  {
    Unpack,
    DigiTrigger,
    LocalReco,
  };

  enum class RecoData
  {
    Digi,
    Cluster,
    Hit,
  };

}  // namespace cbm::algo

CBM_ENUM_DICT(fles::SubsystemIdentifier,
  // CBM detectors
  {"STS", fles::SubsystemIdentifier::STS},
  {"MVD", fles::SubsystemIdentifier::MVD},
  {"RICH", fles::SubsystemIdentifier::RICH},
  {"TRD", fles::SubsystemIdentifier::TRD},
  {"TRD2D", fles::SubsystemIdentifier::TRD2D},
  {"MUCH", fles::SubsystemIdentifier::MUCH},
  {"RPC", fles::SubsystemIdentifier::RPC},
  {"ECAL", fles::SubsystemIdentifier::ECAL},
  {"PSD", fles::SubsystemIdentifier::PSD},

  // Other detectors (experimental)
  {"T0", fles::SubsystemIdentifier::T0},
  {"TRB3", fles::SubsystemIdentifier::TRB3},
  {"HODOSCOPE", fles::SubsystemIdentifier::Hodoscope},
  {"CHERENKOV", fles::SubsystemIdentifier::Cherenkov},

  // FLES (pattern generators)
  {"FLES", fles::SubsystemIdentifier::FLES},
);

CBM_ENUM_DICT(cbm::algo::Step,
  {"Unpack", Step::Unpack},
  {"DigiTrigger", Step::DigiTrigger},
  {"LocalReco", Step::LocalReco}
);

CBM_ENUM_DICT(cbm::algo::RecoData,
  {"Digi", RecoData::Digi},
  {"Cluster", RecoData::Cluster},
  {"Hit", RecoData::Hit}
);

#endif
